# Generated by roxygen2: do not edit by hand

S3method(print,CCFFbinsglm)
S3method(print,CCFFbinspwc)
S3method(print,CCFFbinsqreg)
S3method(print,CCFFbinsreg)
S3method(print,CCFFbinsregselect)
S3method(print,CCFFbinstest)
S3method(summary,CCFFbinsglm)
S3method(summary,CCFFbinspwc)
S3method(summary,CCFFbinsqreg)
S3method(summary,CCFFbinsreg)
S3method(summary,CCFFbinsregselect)
S3method(summary,CCFFbinstest)
export(binsglm)
export(binspwc)
export(binsqreg)
export(binsreg)
export(binsregselect)
export(binstest)
import(ggplot2)
importFrom(matrixStats,colWeightedMeans)
importFrom(matrixStats,colWeightedMedians)
importFrom(quantreg,rq)
importFrom(quantreg,rq.fit)
importFrom(quantreg,rq.wfit)
importFrom(quantreg,summary.rq)
importFrom(sandwich,vcovCL)
importFrom(splines,splineDesign)
importFrom(stats,complete.cases)
importFrom(stats,dnorm)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,model.matrix)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reformulate)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
