% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binsreg-package.R
\docType{package}
\name{binsreg-package}
\alias{binsreg-package}
\title{Binsreg Package Document}
\description{
Binscatter provides a flexible, yet parsimonious way of visualizing and summarizing large data sets
             and has been a popular methodology in applied microeconomics and other social sciences. The binsreg package provides tools for
             statistical analysis using the binscatter methods developed in
             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_AER.pdf}{Cattaneo, Crump, Farrell and Feng (2024)} and
             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2026_RESTAT.pdf}{Cattaneo, Crump, Farrell and Feng (2026)}.
             \code{\link{binsreg}} implements binscatter least squares regression with robust inference and plots, including
             curve estimation, pointwise confidence intervals and uniform confidence band.
             \code{\link{binsqreg}} implements binscatter quantile regression with robust inference and plots, including
             curve estimation, pointwise confidence intervals and uniform confidence band.
             \code{\link{binsglm}} implements binscatter generalized linear regression with robust inference and plots, including
             curve estimation, pointwise confidence intervals and uniform confidence band.
             \code{\link{binstest}} implements binscatter-based hypothesis testing procedures for parametric specifications
             of and shape restrictions on the unknown function of interest.
             \code{\link{binspwc}} implements hypothesis testing procedures for pairwise group comparison of binscatter estimators and plots confidence
             bands for the difference in binscatter parameters between each pair of groups.
             \code{\link{binsregselect}} implements data-driven number of bins selectors for binscatter
             implementation using either quantile-spaced or evenly-spaced binning/partitioning.
             All the commands allow for covariate adjustment, smoothness restrictions, and clustering,
             among other features.

             The companion software article,
             \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2025_Stata.pdf}{Cattaneo, Crump, Farrell and Feng (2025)},
             provides further implementation details and empirical illustration. For related Stata, R and Python packages
             useful for nonparametric data analysis and statistical inference, visit
             \href{https://nppackages.github.io/}{https://nppackages.github.io/}.
}
\references{
Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2024: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_AER.pdf}{On Binscatter}. American Economic Review 114(5): 1488-1514.

Supplemental Appendix for On Binscatter: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2024_AER--Supplemental.pdf}{Supplemental Appendix}.

Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2026: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2026_RESTAT.pdf}{Nonlinear Binscatter Methods}. Review of Economics and Statistics, revise and resubmit.

Supplemental Appendix for Nonlinear Binscatter Methods: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2026_RESTAT--Supplemental.pdf}{Supplemental Appendix}.

Cattaneo, M. D., R. K. Crump, M. H. Farrell, and Y. Feng. 2025: \href{https://nppackages.github.io/references/Cattaneo-Crump-Farrell-Feng_2025_Stata.pdf}{Binscatter Regressions}. Stata Journal 25(1): 3-50.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://nppackages.github.io/}
  \item \url{https://github.com/nppackages/binsreg}
  \item Report bugs at \url{https://github.com/nppackages/binsreg/issues}
}

}
\author{
Matias D. Cattaneo (maintainer). \email{matias.d.cattaneo@gmail.com}.

Richard K. Crump. \email{richard.crump@gmail.com}.

Max H. Farrell. \email{mhfarrell@gmail.com}.

Yingjie Feng. \email{fengyingjiepku@gmail.com}.
}
